;tab4:UTF-8
;	PC-6001/mk2/SR用外付けFDD入出力ルーチン+フォーマット
;
;   PC-6031/6031SR/8031-1/8031-2/80S31/SD6031(WIF) etc.
;
;   1D/1DD 切り替えは手動 -> call Disk60_Change1DD
;   2byte/4byte 転送コマンドはドライブがサポートしていれば自動で使う
;
;	PC66 内蔵 1D は全く構造が違うので Disk66 を使うこと
;
;ロードのみ:510byte 全部入り:763byte
;
;memo.
;サイズを抑えるためロード処理にタイムアウトは設定していない。
;
;  メインシステム側の 8255とディスク側の 8255 で通信を行いコマンドやデータを受け渡す
;	bit  7     6     5     4     3     2     1     0
;  Main	ATN | DAC | RFD | DAV | n/c | DAC | RFD | DAV	Main bit7-4=Write が Disk bit3-0=Readに
;  Disk	n/c | DAC | RFD | DAV | ATN | DAC | RFD | DAV	Disk bit7-4=Write が Main bit3-0=Readに繋がっている
;  IO_FDDCNTWRITT($D3) にはビット位置とセット/リセットを指定して書き込む
;  0000bbbn  bbb=ビット位置, n=0/1
;  たとえば RFD をセットする場合には bit5=1 なので 00001011 となる
;
;	ATN:Attention 		次に送るデータがコマンドであることを示す
;	DAC:Data Accepted	データの受信完了フラグ
;	RFD:Ready for Data	受信準備が出来ていることを示す
;	DAV:Data Valid		送信したデータが有効であることを示す

; HL=ロード/セーブ開始アドレス
; B= セクタ数
; C= ドライブ番号
; D= トラック番号
; E= セクタ番号

	relaxed on

READDISK	equ	$1b67
WRITEDISK	equ	$1b6a

	IFNDEF IO_FDDREAD
IO_FDDREAD		equ	$D0					;FDDからのデータ入力
IO_FDDWRITE		equ	$D1					;FDDへのデータ出力
IO_FDDCNTREAD	equ	$D2					;FDDからの制御信号入力
IO_FDDCNTWRITE	equ	$D3					;FDDへの制御信号出力
	ENDIF

;DISK60_NOSAVE	equ	1					;SAVEを使うときはコメントアウト

;--------------------------------------------------------------------------------------------------

        org     $00c0

Disk60_Entry:
	jp			Disk60_Init				;+0 初回に必ず実行
	jp			Disk60_Load				;+3
		IFDEF DISK60_NOSAVE
	jp			$						;+6
	jp			$						;+9
		ELSE
	jp			Disk60_Save				;+6
	jp			Disk60_Format			;+9
		ENDIF
	jp			Disk60_Change1DD		;+12


;設定された時間が経過すると、cfを立てて帰る
Disk60_timeout:
	or			a
.time1:
	ld			a,0
	dec			a
	ld			(Disk60_timeout.time1+1),a
	ret			nz
.time2:
	ld			a,0
	dec			a
	ld			(Disk60_timeout.time2+1),a
	ret			nz
.time3
	ld			a,1
	dec			a
	ld			(Disk60_timeout.time3+1),a
	ret			nz
	scf
	ret
.reset:
	ld			a,1						;フォーマット時10
	ld			(Disk60_timeout.time3+1),a
	xor			a
	ld			(Disk60_timeout.time1+1),a
	ld			(Disk60_timeout.time2+1),a
	ret

;--------------------------------------------------------------------------------------------------


	org	$0100
	jp	READDISK


;--------------------------------------------------------------------------------------------------
Disk60_Init:
	call		.check					;初回のみ

	ld			a,$91
	out			(IO_FDDCNTWRITE),a
	xor			a						;cmd: Initialize
	jp			Disk60_sendcmd

.check:
	ld			a,$3A
	ld			(Disk60_Init),a			;ld a,(xxxx) ダミーに書き換え

	ld			a,$0B					;Get Memory $07EF,$07F0
	call		Disk60_sendcmd
	jr			c,.skip
	ld			a,$07					;AH
	call		Disk60_sendbyte
	jr			c,.skip
	ld			a,$EF					;AL
	call		Disk60_sendbyte
	jr			c,.skip
	xor			a						;DH
	call		Disk60_sendbyte
	jr			c,.skip
	ld			a,$02					;DL
	call		Disk60_sendbyte
	jr			c,.skip

	call		Disk60_readbyte
	jr			c,.skip
	cp		$40						;P6VW 対策
	jr		z,.x1
	inc			a						;$FFなら、x1のみ SD6031/PC-80S31は $EF が返る
	jr			z,.x1
	call		Disk60_readbyte
	jr			c,.skip
	inc			a						;$FFなら、SD6031
	jr			z,.sd6031
	jr			.x2
.x1:
	call		Disk60_readbyte			;2byte目の残り
.skip:
	ld			hl,Disk60_readdata.patch2
	ld			de,Disk60_readdata.patch1
	ld			bc,Disk60_readdata.end-Disk60_readdata.patch2
	ldir
		IFNDEF DISK60_NOSAVE
	ld			hl,Disk60_senddata.patch2
	ld			de,Disk60_senddata.patch1
	ld			bc,Disk60_senddata.end-Disk60_senddata.patch2
	ldir
		ENDIF
	ld			bc,"1x"
	ret
.x2:
	ld			a,$12
	ld			(Disk60_Load.patch+1),a
		IFNDEF DISK60_NOSAVE
	ld			a,$11
	ld			(Disk60_Save.patch+1),a
		ENDIF
	ld			bc,"2x"
	ret
.sd6031:
	xor			a
	call		Disk60_sendcmd			;ERRORフラグをクリアする
	jr			c,.skip

	ld			a,$88					;x4 コマンドを出力してみる
	call		Disk60_sendcmd
	jr			c,.skip

	ld			a,$06					;Command Status
	call		Disk60_sendcmd
	jr			c,.skip

	call		Disk60_readbyte
	jr			c,.skip
	rrca								;最下位ビットがERROR
	jr			c,.x4					;ERROR時、x4コマンドがある。

	ld			a,$12					;x2 コマンドを出力してみる
	call		Disk60_sendcmd
	jr			c,.skip

	ld			a,$06					;Command Status
	call		Disk60_sendcmd
	jr			c,.skip

	call		Disk60_readbyte
	jr			c,.skip
	rrca								;最下位ビットがERROR
	jr			c,.x2					;ERROR時、x2コマンドがある。
	jr			.x1

.x4:
	ld			a,$88
	ld			(Disk60_Load.patch+1),a
	ld			hl,Disk60_read4data
	ld			(Disk60_Load.loop+1),hl
		IFNDEF DISK60_NOSAVE
	ld			a,$86
	ld			(Disk60_Save.patch+1),a
	ld			hl,Disk60_send4data
	ld			(Disk60_Save.loop+1),hl
		ENDIF
	ld			bc,"4x"
	ret


	org	$01C5
	jp	WRITEDISK


;--------------------------------------------------------------------------------------------------
Disk60_Change1DD:
	ld			a,$17					;cmd: Set Mode
	call		Disk60_sendcmd
	jp			c,Disk60_Init			;cf=1 ならタイムアウトエラー
	ld			a,$0f
	jp			Disk60_sendbyte


;--------------------------------------------------------------------------------------------------
Disk60_Load:
	ld			a,2						;cmd: Read Data
	call		Disk60_sendcmd
	ret			c
	ld			a,17
	sub			e						;a=そのトラックの残りセクタ数
	cp			b						;読み込みセクタ数(1-
	jr			c,.skip
	ld			a,b
.skip:
	call		Disk60_sendbyte
	ret			c
	ld			a,c						;ドライブ番号(0/1)
	call		Disk60_sendbyte
	ret			c
	ld			a,d						;トラック番号(0-39)
	call		Disk60_sendbyte
	ret			c
	ld			a,e						;セクタ番号(1-16)
	call		Disk60_sendbyte
	ret			c

.patch:									;$12=Fast Transfer Buffer / $88=Double High Speed Transfer Read Data
	ld			a,3						;cmd: Transfer Buffer
	call		Disk60_sendcmd
	ret			c

.loop:
	call		Disk60_readdata

	inc			e
	ld			a,e
	cp			17
	jr			z,.next
	djnz		.loop
	or			a						;cf=0 正常終了
	ret

.next:									;次のトラックへ
	inc			d
	ld			e,1
	djnz		Disk60_Load				;cf=0 正常終了
;	or			a
	ret

;--------------------------------------------------------------------------------------------------
Disk60_sendcmd:
	push		af
	ld			a,$0F					;Attention(set)
	out			(IO_FDDCNTWRITE),a
	pop			af

Disk60_sendbyte:						;1byte送信する
	push		af
	call		Disk60_timeout.reset
.wait:
	call		Disk60_timeout
	jr			c,.err
	in			a,(IO_FDDCNTREAD)
	and			%00000010
	jr			z,.wait					;Ready For Data待ち

	ld			a,$0E
	out			(IO_FDDCNTWRITE),a		;Attention(reset)

	pop			af
	out			(IO_FDDWRITE),a			;send data

	ld			a,$09
	out			(IO_FDDCNTWRITE),a		;Data Valid(set)

	call		Disk60_timeout.reset
.wait1:
	call		Disk60_timeout
	ret			c
	in			a,(IO_FDDCNTREAD)
	and			%00000100
	jr			z,.wait1				;Data Accepted待ち

	ld			a,$08
	out			(IO_FDDCNTWRITE),a		;Data Valid(reset)
	call		Disk60_timeout.reset
.wait2:
	call		Disk60_timeout
	ret			c
	in			a,(IO_FDDCNTREAD)
	and			%00000100
	jr			nz,.wait2				;Data Accepted待ち
	ret									;cf=0で正常終了
.err:
	pop			af
	scf
	ret


        org     $0273           ; フォーマット
        rst     $38

;--------------------------------------------------------------------------------------------------
Disk60_readbyte:
	ld			a,$0B
	out			(IO_FDDCNTWRITE),a		;Ready For Data(set)
	call		Disk60_timeout.reset
.wait1:
	call		Disk60_timeout
	ret			c
	in			a,(IO_FDDCNTREAD)
	rrca
	jr			nc,.wait1				;Data Valid=1待ち

	ld			a,$0A
	out			(IO_FDDCNTWRITE),a		;Ready For Data(reset)

	in			a,(IO_FDDREAD)			;Read byte
	push		af

	ld			a,$0D
	out			(IO_FDDCNTWRITE),a		;Data Accepted(set)
	call		Disk60_timeout.reset
.wait2:
	call		Disk60_timeout
	jr			c,.err
	in			a,(IO_FDDCNTREAD)
	rrca
	jr			c,.wait2				;Data Valid=0待ち

	ld			a,$0C
	out			(IO_FDDCNTWRITE),a		;Data Accepted(reset)

	pop			af
	or			a						;cf=0 正常終了
	ret
.err:
	pop			af
	scf
	ret

;--------------------------------------------------------------------------------------------------
;SD6031(WIF)専用4byte読み込み
Disk60_read4data:
	push		bc
	push		de

	ld			a,$93
	out			(IO_FDDCNTWRITE),a		;8255 Mode Set = PortA(In)/B(In)

	ld			a,$0B
	out			(IO_FDDCNTWRITE),a		;Ready For Data(set)
.wait1:
	in			a,(IO_FDDCNTREAD)
	rrca
	jr			nc,.wait1				;Data Valid=1待ち

	ld			a,$0A
	out			(IO_FDDCNTWRITE),a		;Ready For Data(reset)

	ld			b,0						;b=256
	ld			de,($0D << 8) | $0C
.loop:
	ld			c,IO_FDDREAD			;=$D0
.wait2:
	in			a,(IO_FDDCNTREAD)
	rrca
	jr			nc,.wait2				;Data Valid=1待ち

	ini									;(1)
	inc			c
	ini									;(2)
	dec			c

	ld			a,d						;=$0D
	out			(IO_FDDCNTWRITE),a		;Data Accepted(set)
.wait3:
	in			a,(IO_FDDCNTREAD)
	rrca
	jr			c,.wait3				;Data Valid=0待ち

	ini									;(3)
	inc			c
	ini									;(4)

	ld			a,e						;=$0C
	out			(IO_FDDCNTWRITE),a		;Data Accepted(reset)
	jp			nz,.loop

	ld			a,$91
	out			(IO_FDDCNTWRITE),a		;8255 Mode Set = PortA(In)/B(Out)
	pop			de
	pop			bc
	ret


;--------------------------------------------------------------------------------------------------

	org	$02ea           ; 使えるエリアがここまで
        db      $00,$00

        org     $02ec

DISPERR:
        push    bc
        push    de
        push    hl

        ld      bc,40*8*24+14

        ld      hl,ERR_MSG
.lp1:
        ld      a,(hl)
        or      a
        jr      z,.end
        inc     hl
        ld      e,a
        ld      d,$00
        push    hl
        push    bc
        ld      h,b
        ld      l,c

        call    $1ff5

        pop     bc
        inc     bc
        pop     hl
        jr      .lp1

.end:
        pop     hl
        pop     de
        pop     bc
        ret


ERR_MSG:
        db      " DISK ERROR ",$00


        org     $032c           ; 使えるエリアがここまで
        db      $00

        org     $03bf
        db      $ff


	org	$03c0
        phase   $18e9

;--------------------------------------------------------------------------------------------------
Disk60_readdata:
	push		bc
	push		de
	ld			bc,(0 << 8) | IO_FDDREAD
	ld			de,($0A << 8) | $0C
.loop:
	ld			a,$0B
	out			(IO_FDDCNTWRITE),a		;Ready For Data(set)
.wait1:
	in			a,(IO_FDDCNTREAD)
	rrca
	jr			nc,.wait1				;Data Valid=1待ち

	ld			a,d						;=$0A
	out			(IO_FDDCNTWRITE),a		;Ready For Data(reset)

	ini									;(1)

	ld			a,$0D
	out			(IO_FDDCNTWRITE),a		;Data Accepted(set)
.wait2:
	in			a,(IO_FDDCNTREAD)
	rrca								;rrca は zf に変化なし
	jr			c,.wait2				;Data Valid=0待ち
.patch1:
	ini									;(2) read1data はここを詰める
.patch2:
	ld			a,e						;=$0C
	out			(IO_FDDCNTWRITE),a		;Data Accepted(reset)

	jp			nz,.loop				;タイムアウト処理を入れると条件が変わるので注意
	pop			de
	pop			bc
	or			a						;cf=0 正常終了
	ret
.end:


;--------------------------------------------------------------------------------------------------
	IFNDEF DISK60_NOSAVE				;有効にするとセーブとフォーマットが無効になる

Disk60_senddata:
	push		bc
	push		de
	ld			bc,(0 << 8) | IO_FDDWRITE
	ld			de,(4 << 8) | 2

	call		Disk60_timeout.reset
.loop:
	call		Disk60_timeout
	jr			c,.end
	in			a,(IO_FDDCNTREAD)
	and			e						;=2
	jr			z,.loop					;Ready For Data待ち

	outi								;(1)

	ld			a,$09
	out			(IO_FDDCNTWRITE),a		;Data Valid(set)

	call		Disk60_timeout.reset
.wait1:
	call		Disk60_timeout
	jr			c,.end
	in			a,(IO_FDDCNTREAD)
	and			d						;=4
	jr			z,.wait1				;Data Accepted待ち

.patch1:
	outi								;(2) send1data はここを詰める
.patch2:
	ld			a,$08
	out			(IO_FDDCNTWRITE),a		;Data Valid(reset)
	call		Disk60_timeout.reset
.wait2:
	call		Disk60_timeout
	jp			c,.end					;jrはダメ
	in			a,(IO_FDDCNTREAD)
	and			d						;=4
	jr			nz,.wait2				;Data Accepted待ち

	or			b						;a=0
	jp			nz,.loop				;cf=0 正常終了
	pop			de
	pop			bc
	ret
.end:
	pop			de
	pop			bc
	ret


;--------------------------------------------------------------------------------------------------
;SD6031(WIF)専用4byte書き込み
Disk60_send4data:
	push		bc
	push		de

	ld			a,$81
	out			(IO_FDDCNTWRITE),a		;8255 Mode Set = PortA(Out)/B(Out)
	call		Disk60_timeout.reset
.wait1:
	call		Disk60_timeout
	jr			c,.err
	in			a,(IO_FDDCNTREAD)
	and			%00000010
	jr			z,.wait1				;Ready For Data待ち

	ld			bc,(0 << 8) | $D1
	ld			de,(4 << 8) | 9
.loop:
	outi								;(1)
	dec			c
	outi								;(2)
	inc			c

	ld			a,e						;=$09
	out			(IO_FDDCNTWRITE),a		;Data Valid(set)

	call		Disk60_timeout.reset
.wait2:
	call		Disk60_timeout
	jr			c,.err
	in			a,(IO_FDDCNTREAD)
	and			d
	jr			z,.wait2				;Data Accepted待ち

	outi								;(3)
	dec			c
	outi								;(4)
	inc			c

	ld			a,$08
	out			(IO_FDDCNTWRITE),a		;Data Valid(reset)
	call		Disk60_timeout.reset
.wait3:
	call		Disk60_timeout
	jr			c,.err
	in			a,(IO_FDDCNTREAD)
	and			d
	jr			nz,.wait3				;Data Accepted待ち

	or			b						;a=0
	jp			nz,.loop

	ld			a,$91					;cf=0 正常終了
	out			(IO_FDDCNTWRITE),a		;8255 Mode Set = PortA(In)/B(Out)
.err:
	pop			de
	pop			bc
	ret


;--------------------------------------------------------------------------------------------------
Disk60_Save:
.patch:
	ld			a,1						;cmd: Write Data / $11=Fast Write Disk / $86=Double High Speed Write
	call		Disk60_sendcmd
	ret			c
	ld			a,17
	sub			e						;そのトラックの残りセクタ数
	cp			b						;書き込みセクタ数(1-
	jr			c,.skip
	ld			a,b
.skip:
	call		Disk60_sendbyte
	ret			c
	ld			a,c						;ドライブ番号(0/1)
	call		Disk60_sendbyte
	ret			c
	ld			a,d						;トラック番号(0-39)
	call		Disk60_sendbyte
	ret			c
	ld			a,e						;セクタ番号(1-16)
	call		Disk60_sendbyte
	ret			c

.loop:
	call		Disk60_senddata
	ret			c

	inc			e
	ld			a,e
	cp			17
	jr			z,.next
	djnz		.loop
	or			a						;cf=0 正常終了
	ret

.next:									;次のトラックへ
	inc			d
	ld			e,1
	djnz		Disk60_Save
	or			a						;cf=0 正常終了
	ret


;--------------------------------------------------------------------------------------------------
Disk60_Format:

	ENDIF


        dephase

	org	$04bf
        db      $ff


